package com.persagy.common;

import cn.hutool.core.util.StrUtil;
import lombok.Data;
import lombok.EqualsAndHashCode;

import static com.persagy.common.ErrorCodeConstants.SYSTEM_ERROR_B0001;

/**
 * 业务逻辑异常 Exception
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class BusinessException extends RuntimeException {

    private static final long serialVersionUID = 1L;
    /**
     * 业务错误码
     *
     * @see ErrorCodeConstants
     */
    private String code;
    /**
     * 错误提示
     */
    private String message;

    /**
     * 空构造方法，避免反序列化问题
     */
    public BusinessException(Throwable cause) {
        super(cause);
        code = SYSTEM_ERROR_B0001.getCode();
        message = SYSTEM_ERROR_B0001.getMessage();
    }

    public BusinessException(String code, String message, Throwable cause) {
        super(message, cause);
        this.code = code;
        this.message = message;
    }

    public BusinessException(ErrorCode errorCode) {
        super(errorCode.getMessage());
        code = errorCode.getCode();
        message = errorCode.getMessage();
    }

    public BusinessException(ErrorCode errorCode, Throwable cause) {
        super(errorCode.getMessage(), cause);
        code = errorCode.getCode();
        message = errorCode.getMessage();
    }

    public BusinessException(String code, String message) {
        super(message);
        this.code = code;
        this.message = message;
    }

    public static BusinessException wrap(String code, String errorMsgTemplate, Object... args) {
        return new BusinessException(code, StrUtil.format(errorMsgTemplate, args));
    }

    public String getCode() {
        return code;
    }

    public BusinessException setCode(String code) {
        this.code = code;
        return this;
    }

    @Override
    public String getMessage() {
        return message;
    }

    public BusinessException setMessage(String message) {
        this.message = message;
        return this;
    }


}
