package com.persagy.rest;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.R;
import com.persagy.excel.ExcelData;
import com.persagy.excel.ExcelListener;
import com.persagy.excel.ExcelUtils;
import com.persagy.util.DateUtil;
import com.persagy.util.StringUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.function.Consumer;

@ApiOperation(value = "采集/控制接口")
@RestController
@Slf4j
public class RestApi {

    public static boolean iotExcelImport = true;

    @Value("${excel.sleep.millis:1000}")
    private int sleepMillis;

    @ApiOperation(tags = {"iot数据导入"}, value = "终止所有动态导入")
    @PostMapping(value = "/stopImportIot")
    public R importIot() {
        iotExcelImport = false;
        return R.success("终止动态导入");
    }


    @ApiOperation(tags = {"iot数据导入"}, value = "根据配置路径Excel导入")
    @PostMapping(value = "/importIot")
    public R importIot(@RequestBody JSONObject param) {
        String filePath = param.getString("filePath");
        Integer sheetNo = ObjectUtil.isEmpty(param.getInteger("sheetNo")) ? 0 : param.getInteger("sheetNo");
        Integer headRowNumber = ObjectUtil.isEmpty(param.getInteger("headRowNumber")) ? 0 : param.getInteger("headRowNumber");
        Integer consumerBatchSize = ObjectUtil.isEmpty(param.getInteger("consumerBatchSize")) ? 10 : param.getInteger("consumerBatchSize");
        String startTime = param.getString("startTime");
        String endTime = param.getString("endTime");
        String dynamicTime = param.getString("dynamicTime");
        Boolean dynamicTimeEnable = ObjectUtil.isEmpty(param.getBoolean("dynamicTimeEnable")) ? false : param.getBoolean("dynamicTimeEnable");
        return importIotData(null, filePath, null, sheetNo, headRowNumber, consumerBatchSize, startTime, dynamicTimeEnable, dynamicTime, endTime, false);
    }

    /**
     * 导入文件
     *
     * @param simpleFile
     * @param filePath
     * @param sheetNo
     * @param headRowNumber
     * @param consumerBatchSize
     * @return com.yushu.common.R
     * @author lgy
     * @date 2022/4/26 17:48
     */
    @ApiOperation(tags = {"iot数据导入"}, value = "Excel在线导入")
    @PostMapping(value = "/importOnline")
    public R importExcel(
            HttpServletRequest request,
            @RequestParam(value = "file") MultipartFile simpleFile,
            @RequestParam(value = "filePath", required = false) String filePath,
            @RequestParam(value = "sheetNo", defaultValue = "0") Integer sheetNo,
            @RequestParam(value = "sheetNos", defaultValue = "") String sheetNos,
            @RequestParam(value = "headRowNumber", defaultValue = "0") Integer headRowNumber,
            @RequestParam(value = "consumerBatchSize", defaultValue = "10") Integer consumerBatchSize,
            @RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "dynamicTimeEnable", defaultValue = "false") Boolean dynamicTimeEnable,
            @RequestParam(value = "dynamicTime", required = false) String dynamicTime,
            @RequestParam(value = "endTime", required = false) String endTime
    ) throws Exception {
        if(StrUtil.isNotEmpty(sheetNos)){
            String[] sheetNoArray = sheetNos.split(",");
            for (String s : sheetNoArray) {
                int num = NumberUtil.parseInt(s);
                ThreadUtil.execAsync(()->importIotData(simpleFile, filePath, null, num, headRowNumber, consumerBatchSize, startTime, dynamicTimeEnable, dynamicTime, endTime, false));
            }
            return R.success("excel读取中....");
        }
        return importIotData(simpleFile, filePath, null, sheetNo, headRowNumber, consumerBatchSize, startTime, dynamicTimeEnable, dynamicTime, endTime, false);
    }

    public R importIotData(MultipartFile simpleFile, String filePath, InputStream inputStream, Integer sheetNo, Integer headRowNumber, Integer consumerBatchSize, String startTime, Boolean dynamicTimeEnable,
                           String dynamicTime, String endTime, boolean readFromHbase) {
        if (dynamicTimeEnable && StringUtil.isEmpty(dynamicTime)) {
            dynamicTime = DateUtil.formatNow();
        }
        if (ObjectUtil.isNotEmpty(dynamicTime)) {
            if (StrUtil.isEmpty(startTime)) {
                startTime = "20180101000000";
            }
            if (StrUtil.isEmpty(endTime)) {
                endTime = "20500101000000";
            }

            if (dynamicTime.compareTo(startTime) <= 0) {
                //全是动态
                return dynamicRead(filePath, simpleFile, inputStream, sheetNo, null, headRowNumber, ExcelUtils.consumeIot, consumerBatchSize, startTime, endTime, false);
            }
            if (dynamicTime.compareTo(endTime) >= 0) {
                //全是静态
                ExcelListener<T> excelListener1 = new ExcelListener<>();
                excelListener1.setConsumer(ExcelUtils.consumeIot);
                excelListener1.setSleepMillis(sleepMillis);
                excelListener1.setDynamic(false);
                excelListener1.setStartTime(startTime);
                excelListener1.setEndTime(endTime);
                excelListener1.setOnlyReadHeader(false);
                excelListener1.setBatchSize(consumerBatchSize);
                return ExcelUtils.read(filePath, simpleFile, inputStream, sheetNo, null, headRowNumber, excelListener1);
            }
            //startTime--dynamicTime 静态
            ExcelListener<T> excelListener2 = new ExcelListener<>();
            excelListener2.setConsumer(ExcelUtils.consumeIot);
            excelListener2.setDynamic(false);
            excelListener2.setSleepMillis(sleepMillis);
            excelListener2.setStartTime(startTime);
            excelListener2.setEndTime(dynamicTime);
            excelListener2.setOnlyReadHeader(false);
            excelListener2.setBatchSize(consumerBatchSize);
            R<ExcelListener> staticResult = ExcelUtils.read(filePath, simpleFile, null, sheetNo, null, headRowNumber, excelListener2);
            if (staticResult.getIsFail()) {
                return staticResult;
            }
            //dynamicTime--endTime 动态
            return dynamicRead(filePath, simpleFile, inputStream, sheetNo, null, headRowNumber, ExcelUtils.consumeIot, consumerBatchSize, dynamicTime, endTime, readFromHbase);
        }
        ExcelListener<T> excelListener3 = new ExcelListener<>();
        excelListener3.setConsumer(ExcelUtils.consumeIot);
        excelListener3.setDynamic(false);
        excelListener3.setStartTime(startTime);
        excelListener3.setSleepMillis(sleepMillis);
        excelListener3.setEndTime(endTime);
        excelListener3.setOnlyReadHeader(false);
        excelListener3.setBatchSize(consumerBatchSize);
        return ExcelUtils.read(filePath, simpleFile, inputStream, sheetNo, null, headRowNumber, excelListener3);
    }

    public R dynamicRead(String filePath, MultipartFile simpleFile, InputStream inputStream, Integer sheetNo, Class<T> head, Integer headRowNumber, Consumer<ExcelData> consumer,
                         Integer consumerBatchSize, String startTime, String endTime, boolean readFromHbase) {

        ExcelListener<T> excelListener = new ExcelListener<>();
        excelListener.setConsumer(ExcelUtils.consumeIot);
        excelListener.setDynamic(false);
        excelListener.setStartTime(startTime);
        excelListener.setSleepMillis(sleepMillis);
        excelListener.setEndTime(endTime);
        excelListener.setOnlyReadHeader(true);
        excelListener.setBatchSize(consumerBatchSize);
        R<ExcelListener> resultHead = ExcelUtils.read(filePath, simpleFile, inputStream, sheetNo, null, headRowNumber, excelListener);
        if (resultHead.getIsFail()) {
            return resultHead;
        }
        ExcelListener header = resultHead.getData();
        Boolean readAll = header.getReadAll();
        Integer lastRowNumber = header.getLastRowNumber() + 1;
        R<ExcelListener> resultData = R.success(null);
        iotExcelImport = true;
        while (!readAll && iotExcelImport) {
            String now = DateUtil.formatNow();
            if (startTime.compareTo(now) < 0) {
                ExcelListener listener = new ExcelListener<>();
                listener.setDynamic(true);
                listener.setBuildding(header.getBuildding());
                listener.setMeterList(header.getMeterList());
                listener.setFuncidList(header.getFuncidList());
                listener.setDescriptionList(header.getDescriptionList());
                listener.setBatchSize(consumerBatchSize);
                listener.setConsumer(ExcelUtils.consumeIot);
                listener.setStartTime(startTime);
                listener.setEndTime(now);
                listener.setLastRowNumber(lastRowNumber);
                resultData = ExcelUtils.read(filePath, simpleFile, inputStream, sheetNo, null, lastRowNumber, listener);
                if (resultData.getIsFail()) {
                    return resultData;
                }
                readAll = resultData.getData().getReadAll();
                lastRowNumber = resultData.getData().getLastRowNumber();
                startTime = now;
                if (startTime.compareTo(endTime) >= 0) {
                    readAll = true;
                }
            }
            if (!readAll) {
                long timeMillis = NumberUtil.max(DateUtil.betweenTwoTimeMillis(DateUtil.formatNow(), DateUtil.getplusMinutes(now, 5)), 60000);
                ThreadUtil.safeSleep(timeMillis);
            }
        }
        return resultData;
    }


}
